package interpreter;

import static function.builtin.EVAL.eval;

import environment.RuntimeEnvironment;
import error.LispException;
import parser.LispParser;
import sexpression.SExpression;

public class LispInterpreter {

    protected RuntimeEnvironment environment;
    private LispParser parser;

    public LispInterpreter() {
        this.environment = RuntimeEnvironment.getInstance();
    }

    public void interpret() {
        createParser();
        beforeInterpreting();

        for (prompt(); !parser.isEof(); prompt())
            printValueOfNextSExpression();

        afterInterpreting();
    }

    private void createParser() {
        parser = new LispParser(environment.getInput(), environment.getInputName());
    }

    protected void beforeInterpreting() {}

    protected void prompt() {}

    protected void printValueOfNextSExpression() {
        try {
            printValueOfNextSExpressionWithException();
        } catch (LispException e) {
            environment.getErrorManager().handle(e);
        }
    }

    private void printValueOfNextSExpressionWithException() {
        SExpression sExpression = parser.getNextSExpression();
        String result = environment.decorateValueOutput(String.valueOf(eval(sExpression)));

        environment.getOutput().println(result);
        environment.getOutput().flush();
    }

    protected void afterInterpreting() {
        environment.getOutput().println();
        environment.getOutput().flush();
    }

}
